#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/time.h>

#define TESTFILENAME "./testfile"
#define NUMFILES	128

static int	fd[ NUMFILES ];
static char	file_name[NUMFILES][64];

#define	BIGSIZE	(1024 * 1024 * 16)
unsigned char aray[ (unsigned int) BIGSIZE ];
char scmd[ 256 ];

long calcrate( start, end, fsize )

    struct timeval	*start, *end;
    size_t		fsize;

    {
    struct timeval	lapsed;
    long		therate;

#ifdef DEBUGIT
    if (start->tv_sec != end->tv_sec)
	printf("\n\nstart   tv_sec:%10ld      end  tv_sec:%10ld\n", start->tv_sec, end->tv_sec);
    else
	printf("\n\n");
    printf(  "start  tv_usec:%10ld      end tv_usec:%10ld\n", start->tv_usec, end->tv_usec); 
#endif

    if (start->tv_usec > end->tv_usec)
	{
	end->tv_usec += 1000000;
	end->tv_sec--;
	}

    lapsed.tv_usec = end->tv_usec - start->tv_usec;
    lapsed.tv_sec  = end->tv_sec  - start->tv_sec;

#ifdef DEBUGIT
    printf("lap     tv_sec:%10ld\n", lapsed.tv_sec);
    printf("lap    tv_usec:%10ld\n", lapsed.tv_usec); 
#endif

    therate = (long) ((double) fsize / ((double) lapsed.tv_sec + ((double) lapsed.tv_usec / 1000000)) / 1024 * NUMFILES);

    return( therate );
    }

/*
 * MAIN PROGRAM
 *
 */

main( argc, argv )

int argc;
char *argv[];

{

int	i;

size_t	recsize, filesize, ios2do, todo;
ssize_t	iosize;

struct	timeval	first, second;
long	iwrite_rate=0, rewrite_rate=0, read_rate=0, reread_rate=0;

int	c;

for (i=0; i < BIGSIZE; ++i)
    aray[ i ] = 0xde;

for (i=0; i < NUMFILES; ++i)
    sprintf( file_name[ i ], "%s.%d", TESTFILENAME, i );

for (i=0; i < NUMFILES; ++i)
    unlink( file_name[i] );

printf("%12s %12s  %10s %10s %10s %10s\n", "filesize_KB", "recsize_KB", "iwrite", "rewrite", "read", "reread");

/*for (filesize=1024; filesize<=1073741824; filesize*=2) */
for (filesize=1024; filesize<=(1024*1024*16); filesize*=2) 
    for (recsize=1024; (recsize <= 1024*1024); recsize*=2)
	{

	if (recsize > filesize)
	    continue;
	else
	    {
	    ios2do = filesize / recsize;

	    /*
	     * DO INITIAL WRITE
	     */

	    for (i=0; i < NUMFILES; ++i)
		fd[i] = open( file_name[i], O_RDWR|O_CREAT|0x800, 0666);

#ifdef DEBUGIT
	    printf("filesize %10d KB  recsize %6d KB  IOs2do %6d  ", filesize / 1024, recsize / 1024, ios2do);
#else
	    printf("%12d %12d  ", filesize / 1024, recsize / 1024);
#endif
	    fflush(stdout);

	    gettimeofday( &first, 0 );
	    for (i=0; i < NUMFILES; ++i)
		for (todo=0; todo < ios2do; ++todo)
		    iosize=write(fd[i], aray, recsize);
	    gettimeofday( &second, 0 );

	    for (i=0; i < NUMFILES; ++i)
		fsync( fd[i] );
	    for (i=0; i < NUMFILES; ++i)
		close( fd[i] );

	    iwrite_rate = calcrate( &first, &second, filesize );

	    printf("%10ld ", iwrite_rate);
	    fflush(stdout);

	    /*
	     * DO RE-WRITE
	     */

	    for (i=0; i < NUMFILES; ++i)
		fd[i] = open( file_name[i], O_RDWR|0x800, 0);
 
	    gettimeofday( &first, 0 );
	    for (i=0; i < NUMFILES; ++i)
		for (todo=0; todo < ios2do; ++todo)
		    iosize=write(fd[i], aray, recsize);
	    gettimeofday( &second, 0 );

	    for (i=0; i < NUMFILES; ++i)
		fsync( fd[i] );
	    for (i=0; i < NUMFILES; ++i)
		close( fd[i] );

	    rewrite_rate = calcrate( &first, &second, filesize );

	    printf("%10ld ", rewrite_rate);
	    fflush(stdout);

	    /*
	     * DO READ
	     */

	    for (i=0; i < NUMFILES; ++i)
		fd[i] = open( file_name[i], O_RDONLY|0x800, 0);

	    gettimeofday( &first, 0 );
	    for (i=0; i < NUMFILES; ++i)
		for (todo=0; todo < ios2do; ++todo)
		    iosize=read(fd[i], aray, recsize);
	    gettimeofday( &second, 0 );

	    for (i=0; i < NUMFILES; ++i)
		fsync( fd[i] );
	    for (i=0; i < NUMFILES; ++i)
		close( fd[i] );

	    read_rate = calcrate( &first, &second, filesize );

	    printf("%10ld ", read_rate);
	    fflush(stdout);

	    /*
	     * DO REREAD
	     */

	    for (i=0; i < NUMFILES; ++i)
		fd[i] = open( file_name[i], O_RDONLY|0x800, 0);

	    gettimeofday( &first, 0 );
	    for (i=0; i < NUMFILES; ++i)
		for (todo=0; todo < ios2do; ++todo)
		    iosize=read(fd[i], aray, recsize);
	    gettimeofday( &second, 0 );

	    for (i=0; i < NUMFILES; ++i)
		fsync( fd[i] );
	    for (i=0; i < NUMFILES; ++i)
		close( fd[i] );

	    reread_rate = calcrate( &first, &second, filesize );

	    printf("%10ld ", reread_rate);
	    fflush( stdout );

	    for (i=0; i < NUMFILES; ++i)
		unlink( file_name[i] );

	    printf("\n");
	    fflush( stdout );
	    }
	}
}
